diff --git a/src/backend/rewrite/rewriteManip.c b/src/backend/rewrite/rewriteManip.c
index 1c61085a0a7..df96b1bba6a 100644
--- a/src/backend/rewrite/rewriteManip.c
+++ b/src/backend/rewrite/rewriteManip.c
@@ -637,41 +637,58 @@ ChangeVarNodes_walker(Node *node, ChangeVarNodes_context *context)
 	}
 	if (IsA(node, RestrictInfo))
 	{
-		RestrictInfo *rinfo = (RestrictInfo *) node;
-		int			relid = -1;
-		bool		is_req_equal =
-			(rinfo->required_relids == rinfo->clause_relids);
-		bool		clause_relids_is_multiple =
-			(bms_membership(rinfo->clause_relids) == BMS_MULTIPLE);
-
-		if (bms_is_member(context->rt_index, rinfo->clause_relids))
+		RestrictInfo   *rinfo = (RestrictInfo *) node;
+		int				relid = -1;
+		bool			is_req_equal =
+							(rinfo->required_relids == rinfo->clause_relids);
+		bool			is_multiple =
+						(bms_membership(rinfo->clause_relids) == BMS_MULTIPLE);
+
+		if (bms_is_member(context->rt_index, rinfo->clause_relids
+					/*bms_union(rinfo->clause_relids, rinfo->required_relids)*/))
 		{
-			expression_tree_walker((Node *) rinfo->clause, ChangeVarNodes_walker, (void *) context);
-			expression_tree_walker((Node *) rinfo->orclause, ChangeVarNodes_walker, (void *) context);
+			Relids new_clause_relids;
 
-			rinfo->clause_relids =
-				adjust_relid_set(rinfo->clause_relids, context->rt_index, context->new_index);
+			expression_tree_walker((Node *) rinfo->clause,
+								   ChangeVarNodes_walker, (void *) context);
+			expression_tree_walker((Node *) rinfo->orclause,
+								   ChangeVarNodes_walker, (void *) context);
+
+			new_clause_relids = adjust_relid_set(rinfo->clause_relids,
+												 context->rt_index,
+												 context->new_index);
+
+			/* This operation is legal until we remove only baserels */
+			rinfo->num_base_rels -= bms_num_members(rinfo->clause_relids) -
+											bms_num_members(new_clause_relids);
+
+			rinfo->clause_relids = new_clause_relids;
 			rinfo->num_base_rels = bms_num_members(rinfo->clause_relids);
-			rinfo->left_relids =
-				adjust_relid_set(rinfo->left_relids, context->rt_index, context->new_index);
-			rinfo->right_relids =
-				adjust_relid_set(rinfo->right_relids, context->rt_index, context->new_index);
+			rinfo->left_relids = adjust_relid_set(rinfo->left_relids,
+												  context->rt_index,
+												  context->new_index);
+			rinfo->right_relids = adjust_relid_set(rinfo->right_relids,
+												   context->rt_index,
+												   context->new_index);
 		}
 
 		if (is_req_equal)
 			rinfo->required_relids = rinfo->clause_relids;
 		else
-			rinfo->required_relids =
-				adjust_relid_set(rinfo->required_relids, context->rt_index, context->new_index);
+			rinfo->required_relids = adjust_relid_set(rinfo->required_relids,
+													  context->rt_index,
+													  context->new_index);
 
-		rinfo->outer_relids =
-			adjust_relid_set(rinfo->outer_relids, context->rt_index, context->new_index);
-		rinfo->incompatible_relids =
-			adjust_relid_set(rinfo->incompatible_relids, context->rt_index, context->new_index);
+		rinfo->outer_relids = adjust_relid_set(rinfo->outer_relids,
+											   context->rt_index,
+											   context->new_index);
+		rinfo->incompatible_relids = adjust_relid_set(rinfo->incompatible_relids,
+													  context->rt_index,
+													  context->new_index);
 
 		if (rinfo->mergeopfamilies &&
 			bms_get_singleton_member(rinfo->clause_relids, &relid) &&
-			clause_relids_is_multiple &&
+			is_multiple &&
 			relid == context->new_index && IsA(rinfo->clause, OpExpr))
 		{
 			Expr	   *leftOp;
